// log_crate/src/lib.rs

use chrono::Local; // 用于获取当前时间
use colored::*;
use std::cmp::min;
use std::sync::Mutex;
use std::thread;
use std::time::Duration;
// 用于终端彩色输出

/// 模拟当前时间的函数
fn now_str() -> String {
    Local::now().format("%Y-%m-%d %H:%M:%S").to_string()
}

// 通用的格式化打印函数，支持传递多个参数
pub fn printed(format: &str) {
    // 打印彩色的日志信息，带时间戳
    println!("{}", format!("[{}] {}", now_str(), format).green());
}

pub fn printer(format: &str) {
    // 打印彩色的日志信息，带时间戳
    println!("{}", format!("[{}] {}", now_str(), format).red());
}

// 这里作一些新手说明：
// ************************************************************************************************
// $crate::xxx 必须指向一个在当前 crate 中定义的 xxx 函数，或者其他合适的函数实现。
// $crate 确保了宏内部可以正确引用当前 crate 中的函数或类型定义。
// ************************************************************************************************
// #[macro_export] 才能在外部 crate 中使用
// ************************************************************************************************
// $fmt:expr：表示这个参数是一个 Rust 表达式（通常是一个字符串字面量或者 String 类型）。
// $($arg:tt)*：表示后续的多个参数可以是任意的 token（比如字符串字面量、变量、数字等），并且可以是多个（*）。
// 你可以将 fmt 改为任何合法的宏参数模式，如 ident、ty、path 等。具体来说：
//     expr 是指 Rust 表达式，可以是计算得到的值。
//     ident 是标识符，通常用于变量名或函数名。
//     ty 是类型，通常用于类型相关的操作。
//     path 是路径，可以是类似 std::fmt::Arguments 这样的路径。
#[macro_export]
macro_rules! printed {
    // 支持格式化字符串和参数
    ($fmt:expr, $($arg:tt)*) => {
        {
            let formatted = format!($fmt, $($arg)*);  // 格式化字符串
            $crate::printed(&formatted);
        }
    };
}

#[macro_export]
macro_rules! printer {
    // 支持格式化字符串和参数
    ($fmt:expr, $($arg:tt)*) => {
        {
            let formatted = format!($fmt, $($arg)*);  // 格式化字符串
            $crate::printer(&formatted);
        }
    };
}

pub struct MutBuffer {
    buffer: Mutex<Vec<u8>>,
}

impl MutBuffer {
    pub fn new() -> Self {
        MutBuffer {
            buffer: Mutex::new(Vec::new()),
        }
    }

    pub fn push(&self, data: &[u8]) {
        self.buffer.lock().unwrap().extend_from_slice(data);
    }

    pub fn index_of(&self, data: &[u8], start: usize) -> Option<usize> {
        let buffer = self.buffer.lock().unwrap();
        if start + data.len() > buffer.len() || data.is_empty() {
            return None;
        }
        buffer
            .windows(data.len())
            .position(|window| window == data)
            .map(|index| index + start)
    }

    pub fn remove_of(&self, start: usize, len: usize) {
        let mut buffer = self.buffer.lock().unwrap();
        if start >= buffer.len() || len == 0 {
            return;
        }
        let end = min(start + len, buffer.len());
        buffer.drain(start..end);
    }

    pub fn get_len(&self) -> usize {
        let buffer = self.buffer.lock().unwrap();
        buffer.len()
    }

    pub fn clear(&self) {
        let mut buffer = self.buffer.lock().unwrap();
        buffer.clear();
    }
}

pub struct TransProtocol;
const MAX_FRAME_SIZE: usize = 1_048_576;

#[derive(Debug)]
pub struct FrameBuffer {
    pub fid: String,
    pub tid: String,
    pub type_: FrameType,
    pub data: Option<Vec<u8>>,
    pub len: i32,
    pub mark: char,
}

impl FrameBuffer {
    pub fn new() -> Self {
        FrameBuffer {
            fid: String::new(),
            tid: String::new(),
            type_: FrameType::Unknown,
            data: None,
            len: 0,
            mark: '\0',
        }
    }
}

// #[repr(i16)]  // 显示指定枚举底层表示为 i16
#[derive(Debug)]
pub enum FrameType {
    Unknown,
    TypeA,
    TypeB,
}

impl TransProtocol {
    pub fn parse(buffer: &Mutex<Vec<u8>>) -> Option<FrameBuffer> {
        let mut buffer = buffer.lock().unwrap();
        let header = vec![0xFF, 0xFE];
        let tail = vec![0xFF, 0xFF];

        let cur_len = buffer.len();
        if cur_len > MAX_FRAME_SIZE {
            buffer.clear();
            thread::sleep(Duration::new(600, 0));
            return None;
        }

        if let Some(find) = buffer
            .windows(header.len())
            .position(|window| window == &header)
        {
            #[allow(unused_assignments)]
            let mut ttype: i16 = 0;
            #[allow(unused_assignments)]
            let mut mark: char = '\0';
            #[allow(unused_assignments)]
            let mut len: i32 = 0;

            ttype = i16::from_be_bytes(buffer[find + 2..find + 4].try_into().unwrap());
            mark = buffer[find + 4] as char;
            len = i32::from_be_bytes(buffer[find + 6..find + 10].try_into().unwrap());

            let tail_index = find + 10 + len as usize;
            if buffer.len() < tail_index + tail.len() || len < 0 {
                return None;
            }
            if &buffer[tail_index..tail_index + tail.len()] != tail {
                return None;
            }
            let mut frame = FrameBuffer::new();
            if len > 0 {
                frame.data = Some(vec![0; len as usize]);
            }
            frame.len = len;
            frame.fid = String::from_utf8_lossy(&buffer[find + 4..find + 36]).to_string();
            frame.tid = String::from_utf8_lossy(&buffer[find + 36..find + 68]).to_string();
            frame.mark = mark;
            frame.type_ = match ttype {
                1 => FrameType::TypeA,
                2 => FrameType::TypeB,
                _ => FrameType::Unknown,
            };

            // 复制数据
            if let Some(ref mut data) = frame.data {
                data.copy_from_slice(&buffer[find + 68..find + 68 + len as usize]);
            }

            // 删除已解析的部分
            buffer.drain(0..tail_index + 2);
            return Some(frame);
        }
        None
    }

    pub fn pack(buf: &FrameBuffer) -> Option<Vec<u8>> {
        if buf.data.is_none() {
            return None;
        }

        let header = vec![0xFF, 0xFE];
        let tail = vec![0xFF, 0xFF];
        let len = buf.len + 75; // 包含 header 和 tail 的总长度

        let mut out_buf = vec![0u8; len as usize];

        // 填充数据
        out_buf[0..2].copy_from_slice(&header);
        let tt: i16 = match buf.type_ {
            FrameType::TypeA => 1,
            FrameType::TypeB => 2,
            FrameType::Unknown => 0,
        };
        out_buf[2..4].copy_from_slice(&tt.to_le_bytes());
        out_buf[4..5].copy_from_slice(&[buf.mark as u8]);
        if !buf.fid.is_empty() {
            out_buf[5..37].copy_from_slice(buf.fid.as_bytes());
        }
        if !buf.tid.is_empty() {
            out_buf[37..69].copy_from_slice(buf.tid.as_bytes());
        }
        out_buf[69..73].copy_from_slice(&buf.len.to_be_bytes());

        if let Some(ref data) = buf.data {
            out_buf[73..73 + data.len()].copy_from_slice(data);
        }

        out_buf[len as usize - 2..len as usize].copy_from_slice(&tail);
        Some(out_buf)
    }
}
